package com.alcar.aegis.autentication.springsecurity;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.core.userdetails.jdbc.JdbcDaoImpl;

import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.persistence.model.Usuario;
import com.alcar.aegis.service.businesslogic.IRolesService;
import com.alcar.aegis.service.businesslogic.IUsuariosService;

/**
 * 
 * @author alcar
 * 
 *         Esta clase sobreescribe la de springsecurity para recuperar y manejar
 *         el login de los usuarios de la aplicación
 * 
 */
public class AegisAuthenticationProvider extends JdbcDaoImpl {

	@Autowired
	private IUsuariosService usuariosService;
	
	@Autowired
	private IRolesService rolesService;

	@Override
	public UserDetails loadUserByUsername(String username)
			throws UsernameNotFoundException {
		List<UserDetails> users = loadUsersByUsername(username);

		if (users.size() == 0) {
			throw new UsernameNotFoundException(messages.getMessage(
					"AegisDaoImpl.notFound", new Object[] { username },
					"Username {0} not found"));
		}

		UserDetails user = users.get(0); // contains no GrantedAuthority[]

		Set<GrantedAuthority> dbAuthsSet = new HashSet<GrantedAuthority>();

		if (super.getEnableAuthorities()) {
			dbAuthsSet.addAll(loadUserAuthorities(user.getUsername()));
		}

		if (super.getEnableGroups()) {
			dbAuthsSet.addAll(loadGroupAuthorities(user.getUsername()));
		}

		List<GrantedAuthority> dbAuths = new ArrayList<GrantedAuthority>(
				dbAuthsSet);

		addCustomAuthorities(user.getUsername(), dbAuths);

		if (dbAuths.size() == 0) {
			throw new UsernameNotFoundException(messages.getMessage(
					"JdbcDaoImpl.noAuthority", new Object[] { username },
					"User {0} has no GrantedAuthority"));
		}

		return createUserDetails(username, user, dbAuths);
	}

	@Override
	protected List<UserDetails> loadUsersByUsername(String username) {
		List<UserDetails> listaUsuarios = new ArrayList<UserDetails>();
		List<Usuario> usuarios = usuariosService.getByUsername(username);

		for (Usuario user : usuarios) {
			listaUsuarios.add(new User(username, user.gettPassword(), !user
					.getlBloqueado(), true, true,
					!user.getlBloqueado(), AuthorityUtils.NO_AUTHORITIES));
		}
		return listaUsuarios;
	}

	@Override
	protected List<GrantedAuthority> loadUserAuthorities(String username) {
		List<GrantedAuthority> listaRoles = new ArrayList<GrantedAuthority>();
		List<Role> roles = rolesService.getByRolesUsername(username);

		for (Role role : roles) {
			String roleName = super.getRolePrefix() + role.gettNombre();
			listaRoles.add(new SimpleGrantedAuthority(roleName));
		}
		return listaRoles;
	}

	@Override
	protected List<GrantedAuthority> loadGroupAuthorities(String username) {
		return loadUserAuthorities(username);
	}

	@Override
	protected UserDetails createUserDetails(String username,
			UserDetails userFromUserQuery,
			List<GrantedAuthority> combinedAuthorities) {
		String returnUsername = userFromUserQuery.getUsername();

		if (!super.isUsernameBasedPrimaryKey()) {
			returnUsername = username;
		}

		return new User(returnUsername, userFromUserQuery.getPassword(),
				userFromUserQuery.isEnabled(), true, true, userFromUserQuery.isAccountNonLocked(),
				combinedAuthorities);
	}

}
