
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package com.alcar.aegis.faces.convert;


import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.FacesConverter;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * see Javadoc of <a href="http://java.sun.com/javaee/javaserverfaces/1.2/docs/api/index.html">JSF Specification</a>
 *
 * @author Thomas Spiegl (latest modification by $Author: struberg $)
 * @author alcar {Modificado}
 * @version $Revision: 1188415 $ $Date: 2011-10-24 17:13:21 -0500 (Mon, 24 Oct 2011) $
 */
@FacesConverter("com.alcar.aegis.faces.convert.BooleanAfirmativoConverter")
public class BooleanAfirmativoConverter implements Converter
{
	
	private static Log log = LogFactory
			.getLog(BooleanAfirmativoConverter.class);

    private static final String NO = "No";
	private static final String SI = "Si";
	// FIELDS
    public static final String CONVERTER_ID = "com.alcar.aegis.faces.convert.BooleanAfirmativoConverter";
    public static final String STRING_ID = "javax.faces.converter.STRING";
    public static final String BOOLEAN_ID = "javax.faces.converter.BooleanConverter.BOOLEAN";

    // CONSTRUCTORS
    public BooleanAfirmativoConverter()
    {
    }

    // METHODS
    @Override
    public Object getAsObject(FacesContext facesContext, UIComponent uiComponent, String value)
    {
        if (facesContext == null)
        {
            throw new NullPointerException("facesContext");
        }
        if (uiComponent == null)
        {
            throw new NullPointerException("uiComponent");
        }

        if (value != null)
        {
            value = value.trim();
            if (value.length() > 0)
            {
                try
                {
                	if (StringUtils.equalsIgnoreCase(SI, value)) {
                		return Boolean.TRUE;
                	} else if (StringUtils.equalsIgnoreCase(NO, value)) {
                		return Boolean.FALSE;
                	} else {
                		FacesMessage msg = 
                				new FacesMessage("Error al convertir a boolean", 
                						"Error al convertir a boolean");
                			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
                			throw new ConverterException(msg);
                	}
                }
                catch (Exception e)
                {
                	log.error(ExceptionUtils.getFullStackTrace(e));
                	FacesMessage msg = 
            				new FacesMessage("Error al convertir a boolean", 
            						"Error al convertir a boolean");
            			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
            			throw new ConverterException(msg);
                }
            }
        }
        return null;
    }

    @Override
    public String getAsString(FacesContext facesContext, UIComponent uiComponent, Object value)
    {
        if (facesContext == null)
        {
            throw new NullPointerException("facesContext");
        }
        if (uiComponent == null)
        {
            throw new NullPointerException("uiComponent");
        }

        if (value == null)
        {
            return "";
        }
        if (value instanceof String)
        {
            return (String)value;
        }
        try
        {
        	if (Boolean.TRUE.equals(value)) {
        		return SI;
        	} else if (Boolean.FALSE.equals(value)) {
        		return NO;
        	} else {
        		FacesMessage msg = 
        				new FacesMessage("Error al convertir a cadena debe ser true o false", 
        						"Error al convertir a cadena debe ser true o false");
        			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
        			throw new ConverterException(msg);
        	}
        }
        catch (Exception e)
        {
        	log.error(ExceptionUtils.getFullStackTrace(e));
        	FacesMessage msg = 
    				new FacesMessage("Error al convertir a cadena debe ser true o false", 
    						"Error al convertir a cadena debe ser true o false");
    			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
    			throw new ConverterException(msg);
        }
    }
}