package com.alcar.aegis.gui.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.el.ELException;
import javax.el.Expression;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;
import org.richfaces.component.SortOrder;
import org.richfaces.model.Arrangeable;
import org.richfaces.model.ArrangeableState;
import org.richfaces.model.FilterField;

import com.alcar.aegis.service.businesslogic.IPageableService;
import com.google.common.collect.Maps;

public abstract class AbstractArrangeableModel<T extends BaseDescriptor>
		extends AbstractPageableModel<T> implements Arrangeable {

	private static final String ORDENACION = "ORDENACION";
	private static final String ORDENACION_TIPO = "ORDENACION_TIPO";
	private static final String CAMPO_ORDENACION = "sortProperty";
	private static final String ORDEN_ORDENACION = "sortOrder";

	public AbstractArrangeableModel(Class<T> entityClass) {
		super(entityClass);
	}

	private Logger log = Logger.getLogger(AbstractArrangeableModel.class);

	private List<FilterField> filterFields;

	private boolean ordenado = false;

	private Map<String, SortOrder> ordenaciones = new HashMap<String, SortOrder>();
	private Map<String, String> filterValues = Maps.newHashMap();

	private void appendFilters(FacesContext context, Map<String, Object> filtro) {
		if (filterFields != null) {
			for (FilterField filterField : filterFields) {
				String propertyName = getPropertyName(context,
						filterField.getFilterExpression());
				String filterValue = (String) filterField.getFilterValue();
				if (filterValue != null && filterValue.length() != 0) {
					filtro.put(propertyName, filterValue);
				}
			}
		}
	}

	private void appendSorts(FacesContext context, Map<String, Object> filtro) {
		if (ordenaciones.size() > 0) { // Tenemos ordenacion
			for (String campo : ordenaciones.keySet()) {
				filtro.put(ORDENACION, campo);
				if (SortOrder.ascending.equals(ordenaciones.get(campo))) {
					filtro.put(ORDENACION_TIPO, "ASC");
				} else {
					filtro.put(ORDENACION_TIPO, "DESC");
				}
				log.debug("### Guardando propiedad de orden: " + campo + " "
						+ ordenaciones.get(campo));
			}
		}
	}

	private String getPropertyName(FacesContext facesContext,
			Expression expression) {
		try {
			return (String) ((ValueExpression) expression)
					.getValue(facesContext.getELContext());
		} catch (ELException e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			throw new FacesException(e.getMessage(), e);
		}
	}
	
	public Map<String, String> getFilterValues() {
        return filterValues;
    }

	@Override
	public void walk(FacesContext facesContext, DataVisitor visitor,
			Range range, Object argument) {
		if (PhaseId.RENDER_RESPONSE.equals(facesContext.getCurrentPhaseId())) {
			phaseResponse++;
		}
		log.debug("########## [AbstractArrangeableModel::walk] FASE ID: " + facesContext.getCurrentPhaseId() + " ==> "+phaseResponse);
		
		SequenceRange sequenceRange = (SequenceRange) range;

		if (this.cachedItems == null
				|| !areEqualRanges(this.cachedRange, sequenceRange) || (PhaseId.RENDER_RESPONSE.equals(facesContext.getCurrentPhaseId()) && phaseResponse < 2 ) ) {

			getListaPaginada(facesContext, sequenceRange);
	
			}
		for (T item : this.cachedItems) {
			visitor.process(facesContext, item.getId(), argument);
		}
	}

	private void getListaPaginada(FacesContext facesContext,
			SequenceRange sequenceRange) {
		Map<String, Object> filtro = new HashMap<String, Object>();
		appendFilters(facesContext, filtro);
		appendSorts(facesContext, filtro);

		if (sequenceRange != null) {
			int first = sequenceRange.getFirstRow();
			int rows = sequenceRange.getRows();

			filtro.put(IPageableService.PAGE_FIRST, first);
			if (rows > 0) {
				filtro.put(IPageableService.PAGE_ROWS, rows);
			}
		}

		this.cachedRange = sequenceRange;
		this.cachedItems = servicioPaginado.getListaPaginada(filtro);
	}

	@Override
	public int getRowCount() {
		if (this.cachedRowCount == null) {
			Map<String, Object> filtro = new HashMap<String, Object>();
			appendFilters(FacesContext.getCurrentInstance(), filtro);
			cachedRowCount = servicioPaginado.getNumeroRegistrosFiltro(filtro);
		}
		return this.cachedRowCount;
	}

	public void arrange(FacesContext facesContext, ArrangeableState state) {
		if (state != null) {
			this.filterFields = state.getFilterFields();
		}
		if (!ordenado) {
			String campoOrdenacion = getSortProperty();
			SortOrder ordenActual = getSortOrder();// ordenaciones.get(campoOrdenacion);
			log.debug("#### Ordenacion actual: " + campoOrdenacion + " "
					+ ordenActual);
			ordenaciones.clear();
			modifySortProperty(ordenaciones, campoOrdenacion, ordenActual);

			this.cachedItems = null;
			this.cachedRange = null;
			this.cachedRowCount = null;
			ordenado = true;
		}
	}

	private void modifySortProperty(Map<String, SortOrder> orders,
			String sortProperty, SortOrder currentOrder) {
		if ((currentOrder == null) || (currentOrder == SortOrder.ascending)) {
			orders.put(sortProperty, SortOrder.descending);
		} else {
			orders.put(sortProperty, SortOrder.ascending);
		}
	}

	private String getSortProperty() {
		String sortProperty = FacesContext.getCurrentInstance()
				.getExternalContext().getRequestParameterMap()
				.get(CAMPO_ORDENACION);
		return sortProperty;
	}

	private SortOrder getSortOrder() {
		String ordenacion = FacesContext.getCurrentInstance()
				.getExternalContext().getRequestParameterMap()
				.get(ORDEN_ORDENACION);
		SortOrder sortProperty = SortOrder.unsorted;
		if (!StringUtils.isEmpty(ordenacion)) {
			sortProperty = SortOrder.valueOf(ordenacion);
		}
		return sortProperty;
	}

	public Map<String, SortOrder> getOrdenaciones() {
		return ordenaciones;
	}

	public void setOrdenaciones(Map<String, SortOrder> ordenaciones) {
		this.ordenaciones = ordenaciones;
	}

}