package com.alcar.aegis.gui.model;

import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;
import org.apache.log4j.Logger;

import com.alcar.aegis.service.businesslogic.IPageableService;
import com.google.common.collect.Maps;

public abstract class AbstractPageableModel<T extends BaseDescriptor> extends
		ExtendedDataModel<T> {

	private Logger log = Logger.getLogger(AbstractPageableModel.class);

	protected Class<T> entityClass;
	protected SequenceRange cachedRange = null;
	protected Integer cachedRowCount = null;

	protected List<T> cachedItems = null;
	protected Object rowKey;

	protected IPageableService<T> servicioPaginado;

	protected Long phaseResponse = new Long(0);

	public AbstractPageableModel(Class<T> entityClass) {
		super();
		this.entityClass = entityClass;
	}

	protected static boolean areEqualRanges(SequenceRange range1,
			SequenceRange range2) {
		if (range1 == null || range2 == null) {
			return range1 == null && range2 == null;
		} else {
			return range1.getFirstRow() == range2.getFirstRow()
					&& range1.getRows() == range2.getRows();
		}
	}

	@Override
	public Object getRowKey() {
		return rowKey;
	}

	@Override
	public void setRowKey(Object key) {
		this.rowKey = key;
	}

	@Override
	public void walk(FacesContext facesContext, DataVisitor visitor,
			Range range, Object argument) {
		if (PhaseId.RENDER_RESPONSE.equals(facesContext.getCurrentPhaseId())) {
			phaseResponse++;
		}
		log.debug("########## [AbstractPageableModel::walk] FASE ID: "
				+ facesContext.getCurrentPhaseId() + " ==> " + phaseResponse);

		SequenceRange sequenceRange = (SequenceRange) range;

		if (this.cachedItems == null
				|| !areEqualRanges(this.cachedRange, sequenceRange)
				|| (PhaseId.RENDER_RESPONSE.equals(facesContext
						.getCurrentPhaseId()) && phaseResponse < 2)) {

			Map<String, Object> parametros = Maps.newHashMap();

			if (sequenceRange != null) {
				int first = sequenceRange.getFirstRow();
				int rows = sequenceRange.getRows();

				parametros.put(IPageableService.PAGE_FIRST, first);
				if (rows > 0) {
					parametros.put(IPageableService.PAGE_ROWS, rows);
				}
			}

			this.cachedRange = sequenceRange;
			this.cachedItems = servicioPaginado.getListaPaginada(parametros);
		}

		for (T item : this.cachedItems) {
			visitor.process(facesContext, item.getId(), argument);
		}
	}

	@Override
	public int getRowCount() {
		if (this.cachedRowCount == null) {
			cachedRowCount = servicioPaginado.getNumeroRegistros();
		}
		return this.cachedRowCount;
	}

	@Override
	public T getRowData() {
		for (T t : cachedItems) {
			if (t.getId().equals(this.getRowKey())) {
				return t;
			}
		}
		;
		return null;
	}

	@Override
	public int getRowIndex() {
		return -1;
	}

	@Override
	public Object getWrappedData() {
		return null;
	}

	@Override
	public boolean isRowAvailable() {
		return (getRowData() != null);
	}

	@Override
	public void setRowIndex(int rowIndex) {
	}

	@Override
	public void setWrappedData(Object data) {
	}

	public IPageableService<T> getServicioPaginado() {
		return servicioPaginado;
	}

	public void setServicioPaginado(IPageableService<T> servicioPaginado) {
		this.servicioPaginado = servicioPaginado;
	}

}
