package com.alcar.aegis.gui.model.admin;

import java.io.Serializable;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.service.businesslogic.IConexionService;
import com.alcar.aegis.service.businesslogic.IEsquemaService;

@ManagedBean
@RequestScoped
public class EsquemasBean extends BaseBean implements Serializable {

	private static Log log = LogFactory.getLog(EsquemasBean.class);
	private static final String X_ESQUEMA = "xEsquema";
	private static final long serialVersionUID = -6121343390828709164L;
	private static final String[] ESQUEMA_FIELDS = { "tNombre", "tDriverClass", "tUrl" , "tUsuario" , "tEsquema" , "bDblink" };
    
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tNombre", 10);
		MEDIDAS.put("tDriverClass", 25);
		MEDIDAS.put("tUrl", 25);
		MEDIDAS.put("tUsuario", 10);
		MEDIDAS.put("tEsquema", 5);
		MEDIDAS.put("bDblink", 5);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("tNombre", "left");
		ALINEACION.put("tDriverClass", "left");
		ALINEACION.put("tUrl", "left");
		ALINEACION.put("tUsuario", "left");
		ALINEACION.put("tEsquema", "left");
		ALINEACION.put("bDblink", "center");
	};

	@ManagedProperty(value="#{esquemaService}")
	private IEsquemaService esquemaService;
	@ManagedProperty(value="#{conexionService}")
	private IConexionService conexionService;
	
	private Esquema esquema;

	public void nuevo() {
		this.esquema = new Esquema();
	}

	public void editar(Esquema esquema) {
		log.debug("-----> Editando esquema: " + esquema.gettNombre());
		this.esquema = esquema;
	}
	
	public String eliminar(Esquema esquema) {
		log.debug("Eliminando esquema: " + esquema.gettNombre());
		if (esquema != null && esquemaService.deleteEsquema(esquema.getxEsquema()) > 0) {
			addFacesMessageAviso("Esquema eliminado correctamente");
		} else {
			addFacesMessageError("No se ha podido eliminar el esquema");
		}
		return null;
	}
	
	public void guardar() {
		log.debug("-----> Guardando esquema " + esquema.gettNombre());
		if (esquema != null) {
			Long xEsquema = (!StringUtils.isEmpty(getRequestMap().get(X_ESQUEMA)) ? new Long(getRequestMap().get(X_ESQUEMA)) : null);
			esquema.setxEsquema(xEsquema);
			if (esquemaService.saveEsquema(esquema) > 0) {
				addFacesMessageAviso("Esquema guardado correctamente");
			}
		}
	}
	
	public String[] getFields(){
		return ESQUEMA_FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public Esquema getEsquema() {
		if (esquema == null) {
			esquema = new Esquema();
		}
		return esquema;
	}

	public void setEsquemaService(IEsquemaService esquemaService) {
		this.esquemaService = esquemaService;
	}
	
	public void check(Esquema esquema) {
		if (esquema != null) {
			try {
				if (conexionService.comprobarConexion(esquema)) {
					addFacesMessageAviso("Conexión realizada correctamente " + Calendar.getInstance().getTime() + " " + esquema.gettNombre());
				}
			} catch (Exception e) {
				log.error(ExceptionUtils.getFullStackTrace(e));
				addFacesMessageError("Error, no se ha podido conectar "+ Calendar.getInstance().getTime() + " " + esquema.gettNombre(),e.getMessage() );
			}
		}
	}

	public void setConexionService(IConexionService conexionService) {
		this.conexionService = conexionService;
	}
	
}
