package com.alcar.aegis.gui.model.admin;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Parametro;
import com.alcar.aegis.service.businesslogic.IParametrosService;

@ManagedBean
@RequestScoped
public class ParametrosBean extends BaseBean implements Serializable {

	private static final long serialVersionUID = 1561088307667283683L;
	private static Log log = LogFactory.getLog(ParametrosBean.class);
	private static final String X_PARAMETROS = "xParametros";	
	private static final String[] FIELDS = { "tNombre", "tDescripcion", "lActivo" };
    
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tNombre", 15);
		MEDIDAS.put("tDescripcion", 45);
		MEDIDAS.put("lActivo", 10);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("tNombre", "left");
		ALINEACION.put("tDescripcion", "left");
		ALINEACION.put("lActivo", "center");
	};
	
	@ManagedProperty(value="#{parametrosService}")
	private IParametrosService parametrosService;
	
	private Parametro parametro;
	
	public ParametrosBean() {
		log.debug("Creando parametrosBean");
	}

	public void nuevo() {
		log.debug("Creando nuevo parámetro");
		this.parametro = new Parametro();
	}

	public void editar(Parametro parametro) {
		log.debug("-----> Editando parámetro: " + parametro.gettNombre());
		this.parametro = parametro;
	}
	
	public String eliminar(Parametro parametro) {
		log.debug("Eliminando parámetro: " + parametro.gettNombre());
		if (parametro != null && parametrosService.delete(parametro.getxParametros()) > 0) {
			addFacesMessageAviso("Parámetro eliminado correctamente");
		} else {
			addFacesMessageError("No se ha podido eliminar el parámetro");
		}
		return null;
	}
	
	public void guardar() {
		log.debug("-----> Guardando parámetro " + parametro.gettNombre());
		if (parametro != null) {
			Long xParametro = (!StringUtils.isEmpty(getRequestMap().get(X_PARAMETROS)) ? new Long(getRequestMap().get(X_PARAMETROS)) : null);
			parametro.setxParametros(xParametro);
			if (parametrosService.save(parametro) > 0) {
				addFacesMessageAviso("Parámetro guardado correctamente");
			}
		}
	}
	
	public String[] getFields(){
		return FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public Parametro getParametro() {
		if (parametro == null) {
			parametro = new Parametro();
		}
		return parametro;
	}

	public void setParametrosService(IParametrosService parametrosService) {
		this.parametrosService = parametrosService;
	}
	
}
