package com.alcar.aegis.gui.model.admin;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.service.businesslogic.IRolesService;

@ManagedBean
@RequestScoped
public class RolesBean extends BaseBean implements Serializable {

	private static final String X_ROL = "xRol";

	private static Log log = LogFactory.getLog(RolesBean.class);

	private static final long serialVersionUID = -6474217136560527895L;
	private static final String[] ROL_FIELDS = { "tNombre", "tDescripcion" };
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tNombre", 25);
		MEDIDAS.put("tDescripcion", 65);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("tNombre", "left");
		ALINEACION.put("tDescripcion", "left");
	};

	@ManagedProperty(value="#{rolesService}")
	private IRolesService rolesService;
	private Role rol;

	public void nuevo() {
		this.rol = new Role();
	}

	public void editar(Role role) {
		log.debug("-----> Editando role: " + role.gettNombre());
		this.rol = role;
	}
	
	public String eliminar(Role role) {
		log.debug("Eliminando role: " + role.gettNombre());
		if (role != null && rolesService.deleteRole(role.getxRol()) > 0) {
			addFacesMessageAviso("Role eliminado correctamente");
		} else {
			addFacesMessageError("No se ha podido eliminar el rol");
		}
		return null;
	}
	
	public void guardar() {
		log.debug("-----> Guardando role " + rol.gettNombre());
		if (rol != null) {
			Long xRol = (!StringUtils.isEmpty(getRequestMap().get(X_ROL)) ? new Long(getRequestMap().get(X_ROL)) : null);
			rol.setxRol(xRol);
			if (rolesService.saveRole(rol) > 0) {
				addFacesMessageAviso("Rol guardado correctamente");
			}
		}
	}
	
	public String[] getFields(){
		return ROL_FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public Role getRol() {
		if (rol == null) {
			rol = new Role();
		}
		return rol;
	}

	public void setRolesService(IRolesService rolesService) {
		this.rolesService = rolesService;
	}

}
