package com.alcar.aegis.gui.model.admin;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Usuario;
import com.alcar.aegis.service.businesslogic.IUsuariosService;

@ManagedBean
@RequestScoped
public class UsuariosBean extends BaseBean implements Serializable {

	private static Log log = LogFactory.getLog(UsuariosBean.class);
	private static final long serialVersionUID = -3861869196299201311L;

	@ManagedProperty(value="#{usuariosService}")
	private IUsuariosService usuariosService;

	private Usuario usuario;
	private static final String[] USUARIO_FIELDS = {"tNombre", "tApellido1", "tApellido2", "tUsuario", "fAlta", "fBaja", "tEmail", "lBloqueado"};
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tNombre", 10);
		MEDIDAS.put("tApellido1", 5);
		MEDIDAS.put("tApellido2", 5);
		MEDIDAS.put("tUsuario", 5);
		MEDIDAS.put("fAlta", 5);
		MEDIDAS.put("fBaja", 5);
		MEDIDAS.put("tEmail", 10);
		MEDIDAS.put("lBloqueado", 5);
	};
	 
	 private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	 static
		{
		 ALINEACION.put("tNombre", "left");
		 ALINEACION.put("tApellido1", "left");
		 ALINEACION.put("tApellido2", "left");
		 ALINEACION.put("tUsuario", "left");
		 ALINEACION.put("fAlta", "center");
		 ALINEACION.put("fBaja", "center");
		 ALINEACION.put("tEmail", "left");
		 ALINEACION.put("lBloqueado", "center");
		};


	public Usuario getUsuario() {
		if (usuario == null) {
			usuario = new Usuario();
		}
		return usuario;
	}
	
	public void nuevo() {
		usuario = new Usuario();
		usuario.setfAlta(new Date());
		usuario.setlBloqueado(true);
	}

	public void editar(Usuario usuario) {
		log.debug("-----> Editando usuario " + usuario.gettUsuario());
		this.usuario = usuario;
	}
	
	
	public void guardar() {
		log.debug("-----> Guardando usuario " + usuario.getxUsuario());
		if (usuario != null) {
			Long xUsuario = (!StringUtils.isEmpty(getRequestMap().get("xUsuario")) ? new Long(getRequestMap().get("xUsuario")) : null);
			usuario.setxUsuario(xUsuario);
			if (usuariosService.saveUsuario(usuario) > 0) {
				addFacesMessageAviso("Usuario guardado correctamente");
			}
		}
	}

	public String eliminar(Usuario usuario) {
		log.debug("Eliminando usuario");
		if (usuario != null && usuariosService.deleteUsuario(usuario.getxUsuario()) > 0) {
			addFacesMessageAviso("Usuario eliminado correctamente");
		} else {
			addFacesMessageError("No se ha podido eliminar el usuario o roles asociados");
		}
		return null;
	}
	
	public String[] getFields(){
		return USUARIO_FIELDS;
	}

	public IUsuariosService getUsuariosService() {
		return usuariosService;
	}

	public void setUsuariosService(IUsuariosService usuariosService) {
		this.usuariosService = usuariosService;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}

}
