package com.alcar.aegis.gui.model.admin;

import java.io.Serializable;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.bean.SessionScoped;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.persistence.model.Usuario;
import com.alcar.aegis.service.businesslogic.IRolesUsuarioService;

@ManagedBean
@SessionScoped
public class UsuariosRolesBean extends BaseBean implements Serializable {
	
	private static final String ROL = "xRol";

	private static final String USUARIO = "xUsuario";

	private static final long serialVersionUID = -7808605106150260262L;

	private static Log log = LogFactory.getLog(UsuariosRolesBean.class);
	
	@ManagedProperty(value="#{rolesUsuarioService}")
	private IRolesUsuarioService rolesUsuarioService; 
	
	transient private Usuario rolesUsuario = null;
	
	transient private Long xUsuario = null;
	
	transient private List<Role> rolesDisponibles = null;
	
	public Usuario getRolesUsuario() {
		
		return rolesUsuario;
	}

	private void getRolesAsignadosUsuario() {
		log.debug("#### Recuperando roles usuario: " + xUsuario);
		if (xUsuario != null) {
			if (xUsuario != null) {
				rolesUsuario = rolesUsuarioService.getRolesUsuario(xUsuario);
			}
		}
	}
	
	public void agregarRole() {
		log.debug("### Agregar role...");
		try {
			String usuario = getRequestMap().get(USUARIO);
			xUsuario = Long.valueOf(usuario);
			Long rol = Long.valueOf(getRequestMap().get(ROL));
			log.debug("Agregando rol: " + rol + " a usuario: " + xUsuario);
			rolesUsuarioService.addRole(xUsuario, rol);
			getRolesAsignadosUsuario();
			getRolesDisponiblesUsuario();
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			FacesMessage msg = new FacesMessage("Error", "El usuario o el rol falta o no es correcto");
			FacesContext.getCurrentInstance().addMessage(null, msg);
		}
	}
	
	public void eliminar() {
		try {
			String usuario = getRequestMap().get(USUARIO);
			xUsuario = Long.valueOf(usuario);
			Long rol = Long.valueOf(getRequestMap().get(ROL));
			rolesUsuarioService.deleteRolUsuario(xUsuario, rol);
			getRolesAsignadosUsuario();
			getRolesDisponiblesUsuario();
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			FacesMessage msg = new FacesMessage("Error", "El usuario o el rol falta o no es correcto");
			FacesContext.getCurrentInstance().addMessage(null, msg);
		}
	}
	
	public void roles(Long xUsuario) {
		log.debug("Editar roles");
		this.xUsuario = xUsuario;
		getRolesAsignadosUsuario();
	}
	
	public List<Role> getRolesDisponibles() {
		return rolesDisponibles;
	}

	private void getRolesDisponiblesUsuario() {
		if (xUsuario != null) {
			log.debug("Roles disponibles para el usuario: " + xUsuario);
			rolesDisponibles = rolesUsuarioService.getRolesUsuarioDisponibles(xUsuario);
		} else {
			log.warn("### No hay usuario definido");
		}
	}
	
	public void abrirRolesDisponibles(Long xUsuario) {
		log.debug("Abriendo roles disponibles para el usuario: "+ xUsuario);
		this.xUsuario = xUsuario;
		getRolesDisponiblesUsuario();
	}

	public Long getUsuario() {
		return xUsuario;
	}

	public IRolesUsuarioService getRolesUsuarioService() {
		return rolesUsuarioService;
	}

	public void setRolesUsuarioService(IRolesUsuarioService rolesUsuarioService) {
		this.rolesUsuarioService = rolesUsuarioService;
	}
	

}
