package com.alcar.aegis.gui.model.consultas;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Consulta;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.persistence.model.Parametro;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.service.businesslogic.IConsultasService;
import com.alcar.aegis.service.businesslogic.IEsquemaService;
import com.alcar.aegis.service.businesslogic.IParametrosService;
import com.alcar.aegis.service.businesslogic.IRolesService;

@ManagedBean
// @RequestScoped
@SessionScoped
public class ConsultasBean extends BaseBean implements Serializable {
	
	private static final String X_PARAMETRO = "xParametro";
	private static final String X_ROL = "xRol";
	private static Log log = LogFactory.getLog(ConsultasBean.class);
	private static final String X_CONSULTA = "xConsulta";
	private static final long serialVersionUID = -6121343390828709164L;
	private static final String[] CONSULTA_FIELDS = { "tConsulta", "usuUsuario", "lActiva"};
		
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tConsulta", 20);
		MEDIDAS.put("usuUsuario", 20);
		MEDIDAS.put("lActiva", 20);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("tConsulta", "left");
		ALINEACION.put("usuUsuario", "left");
		ALINEACION.put("lActiva", "left");
	};

	@ManagedProperty(value="#{consultasService}")
	private IConsultasService consultasService;
	@ManagedProperty(value="#{esquemaService}")
	private IEsquemaService esquemaService;
	@ManagedProperty(value="#{parametrosService}")
	private IParametrosService parametrosService;
	@ManagedProperty(value="#{rolesService}")
	private IRolesService rolesService;
	
	private Consulta consulta;
	private List<Esquema> esquemas;
	private Long xEsquema;
	private Integer resultado = null;
	
	private List<Parametro> parametros;
	private List<Parametro> parametrosDisponibles;
	
	private List<Role> roles;
	private List<Role> rolesDisponibles;
	
	
	public void nuevo() {
		this.consulta = new Consulta();
		resultado = null;
	}

	public void editar(Long xConsulta) {
		this.consulta = consultasService.getByConsulta(xConsulta);;
		resultado = null;
		log.debug("-----> Editando consulta: " + consulta.gettConsulta());
	}
	
	/* ##### LISTA DE PARAMETROS #### */
	public void parametros(Long xConsulta) {
		if (consulta == null) {
			consulta = new Consulta();
		}
		consulta.setId(xConsulta);
		log.debug("Cargando parámetros de la consulta: " + xConsulta);
		parametros = parametrosService.getParametrosConsulta(xConsulta.intValue());
		log.debug("-----> Recuperado " + parametros.size() + " parametros de la consulta " + xConsulta);
	}
	
	public List<Parametro> getParametros() {
		return parametros;
	}
	
	public void eliminarParametro() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			Long xParametro = (!StringUtils.isEmpty(getRequestMap().get(X_PARAMETRO)) ? new Long(getRequestMap().get(X_PARAMETRO)) : null);
			if(xConsulta == null || xParametro == null) {
				addFacesMessageError("Existe un error en la aplicación no llega el parámetro o la consulta");
			} else {
				parametrosService.eliminarParametroConsulta(xParametro, xConsulta);
				addFacesMessageAviso("Parámetro eliminado correctamente");
				parametros = parametrosService.getParametrosConsulta(xConsulta.intValue());
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el parámetro de la consulta");
		}
	}
	
	public void listaParametrosDisponibles() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			if (consulta == null) {
				consulta = new Consulta();
			}
			consulta.setId(xConsulta);
			log.debug("recuperando parámetros disponibles para la consulta: " + xConsulta);
			parametrosDisponibles = parametrosService.getParametrosDisponibles(xConsulta);
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el parámetro de la consulta");
		}
	}
	
	public List<Parametro> getParametrosDisponibles() {
		return parametrosDisponibles;
	}
	
	public void agregarParametro() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			Long xParametro = (!StringUtils.isEmpty(getRequestMap().get(X_PARAMETRO)) ? new Long(getRequestMap().get(X_PARAMETRO)) : null);
			if(xConsulta == null || xParametro == null) {
				addFacesMessageError("Existe un error en la aplicación no llega el parámetro o la consulta");
			} else {
				parametrosService.agrearParametroConsulta(xParametro, xConsulta);
				addFacesMessageAviso("Parámetro agregado correctamente");
				// Refrescamos las listas de las tablas
				parametros = parametrosService.getParametrosConsulta(xConsulta.intValue());
				parametrosDisponibles = parametrosService.getParametrosDisponibles(xConsulta);
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el parámetro de la consulta");
		}
		
	}
	
	/* ##### FIN LISTA DE PARAMETROS #### */
	
	
	/* ##### ROLES DE LA CONSULTA */
	
	public List<Role> getRoles() {
		return roles;
	}
	
	public List<Role> getRolesDisponibles() {
		return rolesDisponibles;
	}
	
	public void abrirRolesDisponibles() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			if (xConsulta == null) return;
			if (consulta == null) {
				consulta = new Consulta();
			}
			consulta.setId(xConsulta);
			log.debug("Cargando roles disponibles para la consulta: " + xConsulta);
			rolesDisponibles = rolesService.getRolesDisponiblesConsulta(xConsulta);
			log.debug("-----> Recuperado " + roles.size() + " roles disponibles para la consulta " + xConsulta);
		} catch (Exception e) {
				log.error(ExceptionUtils.getFullStackTrace(e));
				addFacesMessageError("Ha ocurrido un problema y no se ha podido cargar la lista de roles disponibles para la consulta");
		}
		
	}
	
	public void roles() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			if (xConsulta == null) return;
			if (consulta == null) {
				consulta = new Consulta();
			}
			consulta.setId(xConsulta);
			log.debug("Cargando roles de la consulta: " + xConsulta);
			roles = rolesService.getRolesByConsulta(xConsulta);
			log.debug("-----> Recuperado " + roles.size() + " roles de la consulta " + xConsulta);
		} catch (Exception e) {
				log.error(ExceptionUtils.getFullStackTrace(e));
				addFacesMessageError("Ha ocurrido un problema y no se ha podido cargar la lista de roles de la consulta");
		}
	}
	
	public void eliminarRol() {
		
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			Long xRole = (!StringUtils.isEmpty(getRequestMap().get(X_ROL)) ? new Long(getRequestMap().get(X_ROL)) : null);
			if(xConsulta == null || xRole == null) {
				addFacesMessageError("Existe un error en la aplicación no llega el rol o la consulta");
			} else {
				rolesService.eliminarRoleConsulta(xRole, xConsulta);
				addFacesMessageAviso("Rol eliminado correctamente");
				// Refrescamos las listas de las tablas
				roles = rolesService.getRolesByConsulta(xConsulta);
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el rol de la consulta");
		}
	}
	
	public void agregarRole() {
		try {
			Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
			Long xRole = (!StringUtils.isEmpty(getRequestMap().get(X_ROL)) ? new Long(getRequestMap().get(X_ROL)) : null);
			if(xConsulta == null || xRole == null) {
				addFacesMessageError("Existe un error en la aplicación no llega el rol o la consulta");
			} else {
				rolesService.agregarRole(xRole, xConsulta);
				addFacesMessageAviso("Rol agregado correctamente");
				// Refrescamos las listas de las tablas
				roles = rolesService.getRolesByConsulta(xConsulta);
				rolesDisponibles = rolesService.getRolesDisponiblesConsulta(xConsulta);
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido asociar el rol de la consulta");
		}
	}
	
	/* ##### FIN ROLES DE LA CONSULTA */
	
	public void eliminar(Consulta consulta) {
		try {
			log.debug("Eliminando consulta: " + consulta.gettConsulta());
			if (consulta != null && consultasService.delete(consulta.getxConsulta()) > 0) {
				addFacesMessageAviso("Consulta eliminado correctamente");
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar la consulta");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar la consulta");
		}
	}
	
	public void guardar() {
		try {
			log.debug("-----> Guardando consulta " + consulta.gettConsulta());
			int registrosActualizados = 0;
			if (consulta != null) {
				Long xConsulta = (!StringUtils.isEmpty(getRequestMap().get(X_CONSULTA)) ? new Long(getRequestMap().get(X_CONSULTA)) : null);
				consulta.setxConsulta(xConsulta);
				consulta.setUsuXUsuario(getSessionBean().getUser().getxUsuario());
				registrosActualizados = consultasService.save(consulta);
				if (registrosActualizados > 0) {
					addFacesMessageAviso("Consulta guardada correctamente");
				} else {
					addFacesMessageError("La consulta no ha sido guardada");
				}
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("La consulta no ha sido guardada", e.getMessage());
		}
	}
	
	public String[] getFields(){
		return CONSULTA_FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public Consulta getConsulta() {
		if (consulta == null) {
			consulta = new Consulta();
		}
		return consulta;
	}
	
	public void setConsultasService(IConsultasService consultasService) {
		this.consultasService = consultasService;
	}
	
	public List<Esquema> getEsquemas() {
		if (esquemas == null) {
			esquemas = esquemaService.getEsquemas();
		}
		return esquemas;
	}
	
	public void probar() {
		try {
			resultado = consultasService.probarConsulta(consulta.gettQuery(), xEsquema);
			addFacesMessageAviso("La consulta se ha ejecutado correctamente: " + resultado + " filas devueltas");
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("La consulta no es correcta o no se puede acceder al servidor", e.getMessage());
		}
	}
	
	public Integer getResultado() {
		return resultado;
	}

	public void setEsquemaService(IEsquemaService esquemaService) {
		this.esquemaService = esquemaService;
	}

	public Long getxEsquema() {
		return xEsquema;
	}

	public void setxEsquema(Long xEsquema) {
		this.xEsquema = xEsquema;
	}

	public IParametrosService getParametrosService() {
		return parametrosService;
	}

	public void setParametrosService(IParametrosService parametrosService) {
		this.parametrosService = parametrosService;
	}

	public IRolesService getRolesService() {
		return rolesService;
	}

	public void setRolesService(IRolesService rolesService) {
		this.rolesService = rolesService;
	}

}
