package com.alcar.aegis.gui.model.procesos;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Consulta;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.persistence.model.Proceso;
import com.alcar.aegis.persistence.model.ProcesoParametros;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.service.businesslogic.IConsultasService;
import com.alcar.aegis.service.businesslogic.IEsquemaService;
import com.alcar.aegis.service.businesslogic.IProcesosService;

@ManagedBean
@SessionScoped
public class ProcesosBean extends BaseBean implements Serializable {
	
	private static final String X_PROCESOPARAMETRO = "xProcesoparametro";
	private static Log log = LogFactory.getLog(ProcesosBean.class);
	private static final String X_PROCESO = "xProceso";
	private static final String X_ESQUEMA = "xEsquema";
	private static final long serialVersionUID = -6121343390828709164L;
	private static final String[] FIELDS = { "tProceso", "fInicioProceso", "lActivo"};
		
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("tProceso", 20);
		MEDIDAS.put("fInicioProceso", 10);
		MEDIDAS.put("lActivo", 10);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("tProceso", "left");
		ALINEACION.put("fInicioProceso", "center");
		ALINEACION.put("lActivo", "center");
	};

	@ManagedProperty(value="#{consultasService}")
	private IConsultasService consultasService;
	@ManagedProperty(value="#{esquemaService}")
	private IEsquemaService esquemaService;
	@ManagedProperty(value="#{procesosService}")
	private IProcesosService procesosService;
	
	private Proceso proceso = new Proceso();
	private List<Esquema> esquemasProceso;
	private List<Esquema> esquemasDisponibles;
	private Long xEsquema;
	
	private ProcesoParametros parametro;
	
	@PostConstruct
	public void construct() {
		log.info("Creando ProcesosBean");
	}
	
	@PreDestroy
	public void destroy() {
		log.info("Destruyendo ProcesosBean");
	}
	
	public void nuevo() {
		this.proceso = new Proceso();
	}

	public void editar(Long xProceso) {
		this.proceso = procesosService.getByProceso(xProceso.intValue());
		log.debug("-----> Editando proceso: " + proceso.gettProceso());
	}
	
	public void eliminar(Proceso proceso) {
		try {
			log.debug("Eliminando proceso: " + proceso.gettProceso());
			if (proceso != null && procesosService.delete(proceso.getxProceso()) > 0) {
				addFacesMessageAviso("Proceso eliminado correctamente");
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el proceso");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el proceso");
		}
	}
	
	public void guardar() {
		try {
			log.debug("-----> Guardando proceso " + proceso.gettProceso());
			if (proceso != null) {
				Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
				proceso.setxProceso(xProceso);
				int registrosActualizados = procesosService.save(proceso);
				if (registrosActualizados > 0) {
					addFacesMessageAviso("Proceso guardado correctamente");
				} else {
					addFacesMessageError("El proceso no ha sido guardado");
				}
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("El proceso no ha sido guardado", e.getMessage());
		}
	}
	
	public String[] getFields(){
		return FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public Proceso getProceso() {
		if (proceso == null) {
			proceso = new Proceso();
		}
		return proceso;
	}
	
	public void setConsultasService(IConsultasService consultasService) {
		this.consultasService = consultasService;
	}
	
	
	public List<Esquema> getEsquemasProceso() {
		if (proceso.getxProceso() != null) {
			esquemasProceso = esquemaService.getEsquemasProceso(proceso.getxProceso());
		}
		return esquemasProceso;
	}
	
	public void eliminarEsquema() {
		try {
			Long xEsquema = (!StringUtils.isEmpty(getRequestMap().get(X_ESQUEMA)) ? new Long(getRequestMap().get(X_ESQUEMA)) : null);
			// Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
			log.debug("Eliminando esquema del proceso: " + proceso.gettProceso());
			if (proceso.getxProceso() != null && xEsquema != null && esquemaService.deleteEsquemaProceso(proceso.getxProceso(), xEsquema) > 0) {
				addFacesMessageAviso("Asociación de esquema eliminada correctamente");
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar la asociación del esquema");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar la asociación del esquema");
		}
	}
	
	public void ejecutarProceso() {
		try {
			final Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
			if (xProceso != null) {
				log.info("Ejecutando proceso: " + xProceso);
				Runnable ejecutarProceso = new Runnable() {
					public void run() {
						procesosService.ejecutarProceso(xProceso);
					}
				};
				Thread hilo = new Thread(ejecutarProceso);
				log.debug("Comenzando el hilo de ejecución del proceso");
				hilo.start();
				addFacesMessageAviso("El proceso está en ejecución, cuando haya finalizado mostrará los resultados en el menú resultados");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Se ha producido un error que impide lanzar el proceso.", e.getMessage());
		}
	}
	
	
	public List<Consulta> getConsultas() {
		try {
			List<Role> rolesUsuario = getSessionBean().getUser().getRoles();
			if (contieneRole(rolesUsuario, "ROL_ADMIN")) {
				return consultasService.getTodasActivas();
			} else {
				return consultasService.getByRoles(rolesUsuario);
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			return null;
		}
	}

	public void setEsquemaService(IEsquemaService esquemaService) {
		this.esquemaService = esquemaService;
	}

	public Long getxEsquema() {
		return xEsquema;
	}

	public void setxEsquema(Long xEsquema) {
		this.xEsquema = xEsquema;
	}

	public void setProcesosService(IProcesosService procesosService) {
		this.procesosService = procesosService;
	}
	
	public Integer getxProceso() {
		return proceso.getxProceso();
	}
	
	public List<Esquema> getEsquemasDisponibles() {
		// proceso.setxProceso(xProceso);
		log.debug("Abriendo esquemas disponibles para proceso " + proceso.getxProceso());
		if (proceso.getxProceso() != null) {
			esquemasDisponibles = esquemaService.getEsquemasDisponibles(proceso.getxProceso());
		}
		return esquemasDisponibles;
	}

	public void addEsquema() {
		log.info("addEsquema");
		try {
			Long xEsquema = (!StringUtils.isEmpty(getRequestMap().get(X_ESQUEMA)) ? Long.parseLong(getRequestMap().get(X_ESQUEMA)) : null);
			Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
			proceso.setxProceso(xProceso);
			log.debug("Asociando esquema al proceso: " + proceso.getxProceso());
			if (proceso.getxProceso() != null && xEsquema != null &&
					esquemaService.saveEsquemaProceso(proceso.getxProceso(), xEsquema) > 0) {
				addFacesMessageAviso("Esquema asociado correctamente");
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido asociar el esquema");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el esquema");
		}
	}
	
	public List<ProcesoParametros> getParametros() {
		try {
			Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
			if (xProceso != null) {
				proceso.setxProceso(xProceso);
			}
			log.debug("Cargando parametros del proceso: " + proceso.getxProceso());
			
			return procesosService.getParamametrosProceso(proceso.getxProceso());
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se han podido cargar los parámetros del proceso");
			return null;
		}
	}
	
	public void configurarParametros() {
		Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
		proceso.setxProceso(xProceso);
		log.debug("Cargando parametros del proceso: " + proceso.getxProceso());
	}
	
	public void editarParametro() {
		try {
			Integer xProcesoParametro = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESOPARAMETRO)) ? new Integer(getRequestMap().get(X_PROCESOPARAMETRO)) : null);
			Integer xProceso = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESO)) ? new Integer(getRequestMap().get(X_PROCESO)) : null);
			proceso.setxProceso(xProceso);
			log.debug("Editando parametro: "+ xProcesoParametro + " del proceso : " + proceso.getxProceso());
			parametro = procesosService.getParamametro(xProcesoParametro);
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema al intentar editar el parámetro");
		}
	}
	
	public ProcesoParametros getProcesoParametro() {
		if (parametro == null) parametro = new ProcesoParametros(); 
		return parametro;
	}
	
	public void modificarParametro() {
		try {
			Integer xProcesoParametro = (!StringUtils.isEmpty(getRequestMap().get(X_PROCESOPARAMETRO)) ? new Integer(getRequestMap().get(X_PROCESOPARAMETRO)) : null);
			log.debug("Editando parametro: "+ xProcesoParametro + " del proceso : " + proceso.getxProceso());
			
			if (procesosService.actualizarParamametro(parametro) > 0) {
				addFacesMessageAviso("Parámetro modificado correctamente");
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido modificar el parámetro");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema al intentar editar el parámetro");
		}
	}

}
