package com.alcar.aegis.gui.model.resultados;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;

import com.alcar.aegis.gui.model.BaseBean;
import com.alcar.aegis.persistence.model.Resultado;
import com.alcar.aegis.service.businesslogic.IResultadosService;
import com.alcar.aegis.utils.SerializarUtil;

@ManagedBean
@SessionScoped
public class ResultadosBean extends BaseBean implements Serializable {
	
	private static final String X_RESULTADO = "xResultado";
	private static Log log = LogFactory.getLog(ResultadosBean.class);
	private static final long serialVersionUID = -6121343390828709164L;
	private static final String[] FIELDS = { "fEjecucion", "proProceso", "esqEsquema"};  // procesoNombre
		
	private static final Map<String, Integer> MEDIDAS = new HashMap<String, Integer>();
	static {
		MEDIDAS.put("fEjecucion", 10);
		MEDIDAS.put("proProceso", 20);
		MEDIDAS.put("esqEsquema", 20);
	};

	private static final Map<String, String> ALINEACION = new HashMap<String, String>();
	static {
		ALINEACION.put("fEjecucion", "center");
		ALINEACION.put("proProceso", "left");
		ALINEACION.put("esqEsquema", "left");
	};

	@ManagedProperty(value="#{resultadosService}")
	private IResultadosService resultadosService;
	
	private Resultado resultado;
	
	List<HashMap<String, Object>> lista;
	private Set<String> columnas;
	private Long tamano;
	
	@PostConstruct
	public void construct() {
		log.info("Creando ResultadosBean");
	}
	
	@PreDestroy
	public void destroy() {
		log.info("Destruyendo ResultadosBean");
	}
	
	public void eliminar(Resultado resultado) {
		try {
			if (resultado != null) {
				log.debug("Eliminando resultado: " + resultado.getxResultado());
				resultadosService.deleteLog(resultado.getxResultado());
				if (resultadosService.delete(resultado.getxResultado()) > 0) {
					addFacesMessageAviso("Proceso eliminado correctamente");
				} else {
					addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el resultado");	
				}
			} else {
				addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el resultado");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			addFacesMessageError("Ha ocurrido un problema y no se ha podido eliminar el resultado");
		}
	}
	
	public String[] getFields(){
		return FIELDS;
	}

	public Map<String, Integer> getMedidas() {
		return MEDIDAS;
	}

	public Map<String, String> getAlineacion() {
		return ALINEACION;
	}
	
	public void setResultadosService(IResultadosService resultadosService) {
		this.resultadosService = resultadosService;
	}
	
	public String mostrar(Resultado resultado) {
		try {
			this.resultado = resultado;
			String ruta = resultadosService.getRutaResultados();
			File ficheroResultado = new File(ruta + resultado.gettFicheroDatos()); 
			InputStream is = new FileInputStream(ficheroResultado);
			lista = SerializarUtil.deserializarLista(is);
			if (lista.size() > 0) {
				columnas = lista.get(0).keySet();
				tamano = new Long(lista.size());
				log.debug("Recuperadas " + tamano + " filas");
			}
		} catch (Exception e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
		}
		return "/page/resultados/resultado";
	}
	
	public Set<String> getColumnas(){
		return columnas;
	}
	
	public Long getTamano(){
		return tamano;
	}
	
	public List<HashMap<String, Object>> getFilasResultado() {
		return lista;
	}

}
