package com.alcar.aegis.service.businesslogic.impl;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.mapper.ConsultasMapper;
import com.alcar.aegis.persistence.model.Consulta;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.service.businesslogic.IConexionService;
import com.alcar.aegis.service.businesslogic.IConsultasService;
import com.alcar.aegis.service.businesslogic.IEsquemaService;
import com.alcar.aegis.service.businesslogic.IPageableService;

@Service
public class ConsultasService implements IConsultasService, IPageableService<Consulta> {
	
	@Autowired
	private ConsultasMapper consultasMapper;
	@Autowired
	private IConexionService conexionService; 
	@Autowired
	private IEsquemaService esquemasService;

	@Override
	public int delete(Long xConsulta) {
		return consultasMapper.deleteByPrimaryKey(xConsulta);
	}
	
	@Override
	public int save(Consulta consulta) {
		if (consulta.getxConsulta() == null) {
			return consultasMapper.insert(consulta);
		} else {
			return consultasMapper.updateByPrimaryKeyWithBLOBs(consulta);
		}
	}

	@Override
	public List<Consulta> getListaPaginada(Map<String, Object> filtro) {
		return consultasMapper.selectFiltradaPaginada(filtro);
	}

	@Override
	public Integer getNumeroRegistros() {
		return consultasMapper.countNumeroRegistros(null);
	}

	@Override
	public Integer getNumeroRegistrosFiltro(Map<String, Object> filtro) {
		return consultasMapper.countNumeroRegistros(filtro);
	}

	@Override
	public Consulta getByConsulta(Long xConsulta) {
		return consultasMapper.selectByPrimaryKeyWithBlobs(xConsulta);
	}
	
	@Override
	public Integer probarConsulta(String consulta, Long xEsquema) throws Exception {
		Esquema esquema = esquemasService.getEsquema(xEsquema);
		return conexionService.probarConsulta(esquema, consulta);
	}

	@Override
	public List<Consulta> getByUsuario(Long xUsuario) {
		return consultasMapper.selectByUsuario(xUsuario);
	}
	
	@Override
	public List<Consulta> getTodasActivas() {
		return consultasMapper.selectAllActivas();
	}
	
	@Override
	public List<Consulta> getByRoles(List<Role> roles) {
		return consultasMapper.selectByRoles(roles);
	}

}
