package com.alcar.aegis.service.businesslogic.impl;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.mapper.EsquemasMapper;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.service.businesslogic.IEsquemaService;
import com.alcar.aegis.service.businesslogic.IPageableService;
import com.google.common.collect.Maps;

@Service
public class EsquemaService implements IEsquemaService, IPageableService<Esquema> {
	
	@Autowired
	private EsquemasMapper esquemasMapper;

	@Override
	public Esquema getByRole(Long xEsquema) {
		return esquemasMapper.selectByPrimaryKey(xEsquema);
	}

	@Override
	public int deleteEsquema(Long xEsquema) {
		return esquemasMapper.deleteByPrimaryKey(xEsquema);
	}

	@Override
	public int saveEsquema(Esquema esquema) {
		if (esquema.getxEsquema() == null) {
			return esquemasMapper.insert(esquema);
		} else {
			return esquemasMapper.updateByPrimaryKey(esquema);
		}
	}

	@Override
	public List<Esquema> getListaPaginada(Map<String, Object> filtro) {
		return esquemasMapper.selectFiltradaPaginada(filtro);
	}

	@Override
	public Integer getNumeroRegistros() {
		return esquemasMapper.countNumeroRegistros(null);
	}

	@Override
	public Integer getNumeroRegistrosFiltro(Map<String, Object> filtro) {
		return esquemasMapper.countNumeroRegistros(filtro);
	}
	
	@Override
	public List<Esquema> getEsquemas() {
		return esquemasMapper.selectFiltradaPaginada(null);
	}

	@Override
	public Esquema getEsquema(Long xEsquema) {
		return esquemasMapper.selectByPrimaryKey(xEsquema);
	}

	@Override
	public List<Esquema> getEsquemasDisponibles(Integer xProceso) {
		return esquemasMapper.selectDisponiblesParaProceso(xProceso);
	}

	@Override
	public List<Esquema> getEsquemasProceso(Integer xProceso) {
		return esquemasMapper.selectEsquemasProceso(xProceso);
	}
	
	@Override
	public Integer deleteEsquemaProceso(Integer xProceso, Long xEsquema) {
		Map<String, Object> parametros = Maps.newHashMap();
		parametros.put("xProceso", xProceso);
		parametros.put("xEsquema", xEsquema);
		return esquemasMapper.deleteEsquemasProceso(parametros);
	}
	
	@Override
	public Integer saveEsquemaProceso(Integer xProceso, Long xEsquema) {
		Map<String, Object> parametros = Maps.newHashMap();
		parametros.put("xProceso", xProceso);
		parametros.put("xEsquema", xEsquema);
		return esquemasMapper.insertEsquemasProceso(parametros);
	}
	
}
