package com.alcar.aegis.service.businesslogic.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.mapper.ConsultasMapper;
import com.alcar.aegis.persistence.mapper.LogMapper;
import com.alcar.aegis.persistence.mapper.ProcesoParametrosMapper;
import com.alcar.aegis.persistence.mapper.ProcesosMapper;
import com.alcar.aegis.persistence.mapper.ResultadosMapper;
import com.alcar.aegis.persistence.model.Consulta;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.persistence.model.Log;
import com.alcar.aegis.persistence.model.Proceso;
import com.alcar.aegis.persistence.model.ProcesoParametros;
import com.alcar.aegis.persistence.model.Resultado;
import com.alcar.aegis.service.businesslogic.IConexionService;
import com.alcar.aegis.service.businesslogic.IEsquemaService;
import com.alcar.aegis.service.businesslogic.IPageableService;
import com.alcar.aegis.service.businesslogic.IProcesosService;
import com.alcar.aegis.utils.SerializarUtil;
import com.google.common.collect.Maps;

@Service
public class ProcesosService implements IProcesosService, IPageableService<Proceso> {
	
	@Autowired
	private ProcesosMapper procesosMapper;
	
	@Autowired
	private ProcesoParametrosMapper parametrosMapper;
	
	@Autowired
	private IConexionService conexionService; 
	
	@Autowired
	private IEsquemaService esquemasService; 
	
	@Autowired
	private ConsultasMapper consultasMapper; 
	
	@Autowired
	private ResultadosMapper resultadoMapper; 
	
	@Autowired
	private LogMapper logMapper; 
	
	@Value("${resultado.path}")
	private String rutaResultados;
	
	private static org.apache.commons.logging.Log logger = LogFactory.getLog(ProcesosService.class);

	@Override
	public List<Proceso> getListaPaginada(Map<String, Object> filtro) {
		return procesosMapper.selectFiltradaPaginada(filtro);
	}

	@Override
	public Integer getNumeroRegistros() {
		return procesosMapper.countNumeroRegistros(null);
	}

	@Override
	public Integer getNumeroRegistrosFiltro(Map<String, Object> filtro) {
		return procesosMapper.countNumeroRegistros(filtro);
	}

	@Override
	public int delete(Integer xProceso) {
		parametrosMapper.deleteByProceso(xProceso);
		return procesosMapper.deleteByPrimaryKey(xProceso);
	}

	@Override
	public int save(Proceso proceso) {
		if (proceso.getxProceso() == null) {
			cargarParametros(proceso.getxProceso(), proceso.getConXConsulta());
			return procesosMapper.insert(proceso);
		} else {
			Proceso procesoAntiguo = procesosMapper.selectByPrimaryKey(proceso.getxProceso());
			if (procesoAntiguo.getConXConsulta() != proceso.getConXConsulta()) {
				parametrosMapper.deleteByProceso(proceso.getxProceso());
			}
			cargarParametros(proceso.getxProceso(), proceso.getConXConsulta());
			return procesosMapper.updateByPrimaryKey(proceso);
		}
	}

	@Override
	public Proceso getByProceso(Integer xProceso) {
		return procesosMapper.selectByPrimaryKey(xProceso);
	}
	
	@Override
	public List<ProcesoParametros> getParamametrosProceso(Integer xProceso) {
		return parametrosMapper.selectByParamametrosProceso(xProceso);
	}
	
	private void cargarParametros(Integer xProceso, Integer xConsulta) {
		if (xProceso == null || xConsulta == null) return;
		Map<String, Object> record = Maps.newHashMap();
		record.put("xProceso", xProceso);
		record.put("xConsulta", xConsulta);
		parametrosMapper.cargarParametros(record);
	}

	@Override
	public ProcesoParametros getParamametro(Integer xProcesoParametro) {
		return parametrosMapper.selectByPrimaryKeyConDescripcion(xProcesoParametro);
	}

	@Override
	public int actualizarParamametro(ProcesoParametros parametro) {
		return parametrosMapper.updateValorByPrimaryKey(parametro);
	}
	
	@Override
	public int ejecutarProceso(Integer xProceso) {
		// Obtener proceso con esquemas
		List<Esquema> esquemas = esquemasService.getEsquemasProceso(xProceso);
		// obtener valores parametros y sustituir
		List<ProcesoParametros> parametrosProceso = parametrosMapper.selectByParamametrosProceso(xProceso);
		Map<String, Object> parametrosConsulta = prepararParametros(parametrosProceso);
		Proceso proceso = procesosMapper.selectByPrimaryKey(xProceso);
		Consulta consulta = consultasMapper.selectByPrimaryKeyWithBlobs(new Long(proceso.getConXConsulta()));
		for (Esquema esquema : esquemas) {
			try {
				List<Map<String, Object>> listaFilas = conexionService.ejecutarConsulta(esquema, consulta.gettQuery(), parametrosConsulta);
				logger.info("Consulta ejecutado: " + listaFilas.size() + " filas");
				
				String separador = System.getProperty("file.separator");
				String fichero = separador + xProceso + separador + "log_esq"+esquema.getId() +"_"+ convertirDate2String(new Date()) + ".ser";
				File carpetaObjetos = new File(rutaResultados + separador + xProceso);
				File ficheroObjetos = new File(rutaResultados + fichero);
				FileUtils.forceMkdir(carpetaObjetos);
				OutputStream os = new FileOutputStream(ficheroObjetos);
				SerializarUtil.serializarLista(listaFilas, os);
				Resultado resultado = new Resultado();
				resultado.setEsqXEsquema(esquema.getxEsquema().intValue());
				resultado.setfEjecucion(new Date());
				resultado.setProXProceso(proceso.getxProceso());
				resultado.settFicheroDatos(fichero);
				resultadoMapper.insert(resultado);
				
			} catch (Exception e) {
				String pilaExcepcion = ExceptionUtils.getFullStackTrace(e);
				logger.error(e);
				
				Resultado resultado = new Resultado();
				resultado.setEsqXEsquema(esquema.getxEsquema().intValue());
				resultado.setfEjecucion(new Date());
				resultado.setProXProceso(proceso.getxProceso());
				resultado.settFicheroDatos(null);
				resultadoMapper.insert(resultado);
				
				String separador = System.getProperty("file.separator");
				
				String fichero = rutaResultados + separador + xProceso + separador + "log_esq"+esquema.getId() +"_"+ convertirDate2String(new Date()) + ".log";
				File ficheroLog = new File(fichero);
				try {
					FileUtils.writeStringToFile(ficheroLog, pilaExcepcion);
				} catch (IOException e1) {
					logger.error(ExceptionUtils.getFullStackTrace(e1));
				}
				Log logResultado = new Log();
				logResultado.setResXResultado(resultado.getId());
				logResultado.settFicheroLog(fichero);
				logMapper.insert(logResultado);
			}
		}
		// dentro de cada lanzamiento, guardar el resultado en bruto
		// Guardar el log en la parte de logs
		return 0;
	}
	
	private Map<String, Object> prepararParametros(List<ProcesoParametros> parametros) {
		Map<String, Object> parametrosParaConsulta = Maps.newHashMap();
		for (ProcesoParametros procesoParametros : parametros) {
			String nombre = procesoParametros.getParametro().gettNombre();
			String valor = procesoParametros.gettValor();
			nombre = nombre.replace(":", "");
			parametrosParaConsulta.put(nombre, valor);
		}
		return parametrosParaConsulta;
	}
	
	private String convertirDate2String(Date date)
	{
		if (date != null)
		{
			SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
			return df.format(date);
		}
		else
		{
			return null;
		}
	}

}
