package com.alcar.aegis.service.businesslogic.impl;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.mapper.RolesMapper;
import com.alcar.aegis.persistence.model.Role;
import com.alcar.aegis.persistence.model.RolesConsultas;
import com.alcar.aegis.service.businesslogic.IPageableService;
import com.alcar.aegis.service.businesslogic.IRolesService;

@Service
public class RolesService implements IRolesService, IPageableService<Role> {
	
	@Autowired
	private RolesMapper rolesMapper;

	@Override
	public List<Role> getByRolesUsername(String username) {
		
		return rolesMapper.selectByRolesUsername(username);
	}

	@Override
	public List<Role> getListaPaginada(Map<String, Object> filtro) {
		return rolesMapper.selectFiltradaPaginada(filtro);
	}

	@Override
	public Integer getNumeroRegistros() {
		return rolesMapper.countNumeroRegistros(null);
	}

	@Override
	public Integer getNumeroRegistrosFiltro(Map<String, Object> filtro) {
		return rolesMapper.countNumeroRegistros(filtro);
	}

	@Override
	public Role getByRole(Long xRole) {
		return rolesMapper.selectByPrimaryKey(xRole);
	}

	@Override
	public int deleteRole(Long xRol) {
		return rolesMapper.deleteByPrimaryKey(xRol);
	}

	@Override
	public int saveRole(Role rol) {	
		if (rol.getxRol() == null) {
			return rolesMapper.insert(rol);
		} else {
			return rolesMapper.updateByPrimaryKey(rol);
		}
	}

	@Override
	public List<Role> getRolesByConsulta(Long xConsulta) {
		return rolesMapper.selectRolesByConsulta(xConsulta);
	}

	@Override
	public int eliminarRoleConsulta(Long xRole, Long xConsulta) {
		RolesConsultas roleConsulta = new RolesConsultas();
		roleConsulta.setRolXRol(xRole);
		roleConsulta.setConXConsulta(xConsulta.intValue());
		return rolesMapper.deleteByRoleConsulta(roleConsulta) ;
	}

	@Override
	public List<Role> getRolesDisponiblesConsulta(Long xConsulta) {
		return rolesMapper.selectRolesDisponiblesConsulta(xConsulta);
	}

	@Override
	public int agregarRole(Long xRole, Long xConsulta) {
		RolesConsultas roleConsulta = new RolesConsultas();
		roleConsulta.setRolXRol(xRole);
		roleConsulta.setConXConsulta(xConsulta.intValue());
		return rolesMapper.insertRoleConsulta(roleConsulta) ;
	}	

}
