package com.alcar.aegis.service.businesslogic.impl;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.mapper.RolesUsuarioMapper;
import com.alcar.aegis.persistence.mapper.UsuariosMapper;
import com.alcar.aegis.persistence.model.Usuario;
import com.alcar.aegis.service.businesslogic.IPageableService;
import com.alcar.aegis.service.businesslogic.IUsuariosService;

@Service
public class UsuariosService implements IUsuariosService, IPageableService<Usuario> {

	private static final String PASS_CODED = "****";

	@Autowired
	private UsuariosMapper usuarioMapper;
	
	@Autowired
	private RolesUsuarioMapper rolesUsuarioMapper;

	@Override
	public List<Usuario> getAllUsers() {
		return usuarioMapper.selectAll();
	}

	@Override
	public List<Usuario> getByUsername(String username) {
		return usuarioMapper.selectByUsername(username);
	}

	@Override
	public Usuario getUsuario(Long xUsuario) {
		return usuarioMapper.selectByPrimaryKey(xUsuario);
	}
	
	@Override
	public int deleteUsuario(Long xUsuario) {
		rolesUsuarioMapper.deleteRolByUser(xUsuario);
		return usuarioMapper.deleteByPrimaryKey(xUsuario);
	}
	
	@Override
	public Usuario getUsuarioSinPassword(Long xUsuario) {
		return usuarioMapper.selectByPrimaryKeySinPassword(xUsuario);
	}

	@Override
	public int saveUsuario(Usuario usuario) {
		if (StringUtils.equals(usuario.gettPassword(), PASS_CODED)) {
			usuario.settPassword(null);
		}
			
		if (usuario.getxUsuario() == null) {
			// TODO agregar rol por defecto
			return usuarioMapper.insert(usuario);
		} else {
			return usuarioMapper.updateByPrimaryKeyPasswordSelective(usuario);
		}
	}

	@Override
	public List<Usuario> getListaPaginada(Map<String, Object> filtro) {
		return usuarioMapper.selectFiltradaPaginada(filtro);
	}

	@Override
	public Integer getNumeroRegistros() {
		return usuarioMapper.countNumeroRegistros(null);
	}

	@Override
	public Integer getNumeroRegistrosFiltro(Map<String, Object> filtro) {
		return usuarioMapper.countNumeroRegistros(filtro);
	}
	
	

}
